#
# -*- coding: utf-8 -*-

'''General things all querytool scripts need.
It also adjusts the sys path via biocase.adjustpath.py .
This module should be executed via execfile('ABS PATH TO HERE').
'''

# import cgitb; cgitb.enable()
import cgi, os, sys
import logging, biocase.initlogs
biocase.initlogs.initWebappLogging()

from biocase.querytool.preferences import getPreferences, QTPreferences
from biocase.querytool.filter import AndClass, createFilterObjFromString, LikeClass, EqualsClass, LesserClass, GreaterClass, binaryCopObjectFactory, getOptimizedFilterObj


def createFilter(form, schemaObj, LOPClass=AndClass):
    class paraClass:
        '''simple class to handle parameters'''
        def __init__(self):
            self.conceptObj = None
            self.value = None
            self.copType = '~'
    # build filter object
    filterObj = LOPClass()
    # first check if their is a filterstring
    if 'filter' in form:
        oldFilter = createFilterObjFromString(form.getvalue('filter'), schemaObj)
        log.info("OLD FILTER FROM STRING '%s' BECOMES '%s'" % (form.getvalue('filter'), str(oldFilter)))
        log.debug("  AS XML: %s" % oldFilter._reprXML())
        filterObj.addOperand(oldFilter)

    else:
        log.info('NO OLD FILTER FOUND')
    # then check for additional operands
    args = {}
    for para in list(form.keys()):
        # make underscores spaces again as in the prefs object
        if para[:3] == 'con':
            label = para[3:]
            if label in args:
                # the parameter tuple already exists
                args[label].conceptObj = schemaObj.getConcept(label)
                args[label].value = form.getvalue(para)
            else:
                # create new para object
                pObj = paraClass()
                pObj.conceptObj = schemaObj.getConcept(label)
                pObj.value = form.getvalue(para)
                args[label] = pObj
        if para[:3] == 'cop':
            label = para[3:]
            if label in args:
                # the parameter tuple already exists
                args[label].copType = form.getvalue(para)
            else:
                pObj = paraClass()
                pObj.copType = form.getvalue(para)
                args[label] = pObj
    log.info("FILTER ARGS: %s" % str(args))
    # remove empty args (Value=None)
    for label, arg in list(args.items()):
        if arg.value is None or len(arg.value) == 0:
            del args[label]
    log.debug("FILTER ARGS CLEANED: %s" % str(args))
    # create new COP operands
    copclasses = {'~': LikeClass, '=': EqualsClass, '<': LesserClass, '>': GreaterClass}
    for label, pObj in list(args.items()):
        copObj = binaryCopObjectFactory(CopClass=copclasses[pObj.copType], conceptObj=pObj.conceptObj, val=pObj.value)
        log.debug(copObj)
        filterObj.addOperand(copObj)
    return getOptimizedFilterObj(filterObj)


def logDiagnostics(diagnosticsList):
    log.debug("WRAPPER DIAGNOSTICS:")
    for d in diagnosticsList:
        log.debug(d[2])


def transformXML(xml, xslLoc, paras=None):
    log.info("APPLYING STYLESHEET %s" % xslLoc)
    try:
        import lxml.etree as ET
        dom = ET.fromstring(xml)
        xslt = ET.parse(xslLoc)
        transform = ET.XSLT(xslt)
        if paras:
            return transform(dom, dsa=ET.XSLT.strparam(paras['dsa']), unitlink=ET.XSLT.strparam(paras['unitlink']))
        else:
            return transform(dom)
    except:
        log.exception("ERROR when applying the stylesheet")


def printOverHTTP(templ):
    try:
        s = 'Content-Type: text/html\n\n' + str(templ)
        sys.stdout.buffer.write(s.encode('utf8'))
    except:
        log.exception("Couldnt print template.")


# get global project configurations
cfg = biocase.configuration.Cfg()
# debugging to file
log = logging.getLogger("webapp.querytool")
log.info("--- NEW QUERYTOOL REQUEST ---")

# dirs used with the querytool
configDir = os.path.join(cfg.configurationLocator, 'querytool') # querytool config dir
form = biocase.configuration.sanitiseFieldStorage(cgi.FieldStorage())

# check datasource alias (DSA) to parse preferences etc.
if 'dsa' in form:
    dsa = form.getvalue('dsa')
    wrapper_url = cfg.getAccessPoint(dsa)
    prefs = getPreferences(dsa, cfg)
else:
    dsa = None
    wrapper_url = None
    prefs = QTPreferences()
    # check for skin to use:
    if 'skin' in form:
        prefs.skin = form.getvalue('skin')

# allow overriding of wrapper url
if 'wrapper_url' in form:
    wrapper_url = form.getvalue('wrapper_url')

# default directory for HTML templates
skinDir = os.path.join(configDir, 'skins', 'default')
# use custom skin?
if prefs.skin is not None:
    fn = os.path.join(configDir, 'skins', prefs.skin)
    if os.path.isdir(fn):
        skinDir = fn
    else:
        log.warn("Cannot find skin %s" % prefs.skin)

log.debug(str(prefs))
# which schema to use?
if 'schema' in form:
    schemaObj = prefs.getSchema(form.getvalue('schema'))
    if schemaObj is None:
        raise "The requested schema %s is not supported by this datasource." % form.getvalue('schema')
else:
    schemaObj = prefs.defaultSchema
schema = None
if schemaObj is not None:
    schema = schemaObj.NS

# was there a security role passed?
if 'role' in form:
    security_role = form.getvalue('role')
else:
    security_role = None

# for debugging:
log.info("WRAPPER URL: %s" % wrapper_url)
log.info("DSA: %s" % dsa)
log.info("SECURITY ROLE: %s" % security_role)
log.info("SCHEMA: %s" % schema)
log.info("SKIN: %s" % prefs.skin)
log.debug("SkinDir: %s" % skinDir)
log.debug(prefs)
