# -*- coding: utf8 -*-
'''
DB MODULE - generic ODBC
*******************************
***  DB MODULE - generic ODBC
*******************************
$RCSfile: dbmod_odbc.py,v $
$Revision: 1627 $
$Author: j.holetschek $
$Date: 2021-05-25 15:06:16 +0200 (Di, 25. Mai 2021) $

The biocase.wrapper database module to access generic ODBC datasources.
You will need the mxODBC module to connect to a DSN via ODBC.
'''

from biocase.wrapper.errorclasses import DbModuleMissingError, ConnectionError
from biocase.wrapper.sql.base import sqlBase
from biocase.wrapper.dbmod.base_dbmod import DBBasemodule

import logging
log = logging.getLogger("pywrapper.dbmod_odbc")

# uses inherited ANSI SQL apart from the following.
sqlBase.templates['attribute'] = '[#1].[#2]'
sqlBase.templates['table'] = '[#1]'
sqlBase.templates['join'] = '(#1 LEFT JOIN #2 ON #3)'
sqlBase.templates['select-limit'] = 'SELECT DISTINCT TOP #3 #1 FROM #2'


class DBmodule(DBBasemodule):
    __dbmod__ = 'MS Excel module using pyODBC'

    def createDBconnection(self):
        try:
            import pyodbc
        except:
            raise DbModuleMissingError('pyODBC')

        self.__class__.__version__ = pyodbc.version
        loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.database, self.psfObj.user, self.psfObj.password)
        try:
            # autocommit needs to be true for Excel
            return pyodbc.connect(loginParas, autocommit=True)
        except Exception as ex:
            log.exception(ex)
            raise ConnectionError

    def getTables(self):
        try:
            obj = self.conn.cursor()
            return [t[2].strip("'") for t in obj.tables()]
        except:
            log.exception("Couldn't retrieve table info")

    def getColumns(self, table_name):
        try:
            obj = self.conn.cursor()
            return [c[3] for c in obj.columns(table=table_name)]
        except:
            log.exception("Couldn't retrieve column info for table " + table_name)

    def hasMetaInfo(self):
        return True
