# -*- coding: utf8 -*-
'''
DB MODULE - generic ODBC
*******************************
***  DB MODULE - generic ODBC
*******************************
$RCSfile: dbmod_odbc_db2.py,v $
$Revision: 1627 $
$Author: j.holetschek $
$Date: 2021-05-25 15:06:16 +0200 (Di, 25. Mai 2021) $

The biocase.wrapper database module to access generic ODBC datasources.
You will need the mxODBC module to connect to a DSN via ODBC.
'''

import os
from biocase.wrapper.errorclasses       import *
from biocase.tools.various_functions    import unique
from biocase.wrapper.typeconverter      import DBtransformerClass
from biocase.wrapper.sql.operators      import getWhereObjTables
from biocase.wrapper.sql.base           import sqlBase
from biocase.wrapper.sql.select         import sqlSelect
from biocase.wrapper.dbmod.base_dbmod   import DBBasemodule

import logging
log = logging.getLogger("pywrapper.dbmod")


# uses inherited ANSI SQL

#The only difference in DB2 with ANSI SQL is the way to limit the number
#of records, you must use the FETCH FIRST #3 ROWS ONLY clause
sqlBase.templates['select-limit'] = 'SELECT DISTINCT #1 FROM #2 FETCH FIRST #3 ROWS ONLY'
        

# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
# ---------------------------------------------------------------------------------
class DBmodule(DBBasemodule):
    __version__ = ''
    __dbmod__ = 'DB2 ODBC module using '
    

    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def __init__(self, psfObj):
                # call createDBconnection method
        DBBasemodule.__init__(self, psfObj)
        
    # ------------------   FUNCTION OVERLOAD OF BASE DBMOD MODULE   ------------------
    # --------------------------------------------------------------------------------
    def createDBconnection(self):
        try:
            import pyodbc as odbc
            self.__class__.__dbmod__ += "pyODBC"
        except:
            try:
                import ceODBC as odbc
                self.__class__.__dbmod__ += "ceODBC"
            except:
                raise DbModuleMissingError('ceODBC/pyODBC')
        loginParas = "DSN=%s;UID=%s;PWD=%s" % (self.psfObj.database, self.psfObj.user ,self.psfObj.password )
        conn = odbc.connect(loginParas)
        return conn