# -*- coding: utf8 -*-
'''
DB MODULE - ODBC Firebird
*******************************
***  DB MODULE - ODBC Firebird
*******************************
$RCSfile: dbmod_odbc_firebird.py,v $
$Revision: 1627 $
$Author: j.holetschek $
$Date: 2021-05-25 15:06:16 +0200 (Di, 25. Mai 2021) $

The biocase.wrapper database module to access Firebird trough ODBC datasources.
You will need the mxODBC module to connect to a DSN via ODBC.
'''

from biocase.wrapper.sql.base import sqlBase
from biocase.wrapper.dbmod.dbmod_odbc import DBmodule

import logging
log = logging.getLogger("pywrapper.dbmod_odbc_firebird")

# Reset defaults from base template
sqlBase.templates['attribute'] = '#1.#2'
sqlBase.templates['table'] = '#1'
sqlBase.templates['join'] = '#1 LEFT JOIN #2 ON #3'

# no alias tag
sqlBase.templates['alias'] = '#1  #2'

# OFFSET: SELECT FIRST x SKIP y * FROM
sqlBase.templates['select-limit'] = 'SELECT FIRST #3 DISTINCT #1 FROM #2'
sqlBase.templates['select-limit-offset'] = 'SELECT FIRST #3 SKIP #4 DISTINCT #1 FROM #2'


class DBmodule(DBmodule):
    __dbmod__ = 'Firebird module using pyODBBC'

    def getTables(self):
        tables = []
        self.c = self.conn.cursor()
        try:
            self.c.execute("select rdb$relation_name from rdb$relations where rdb$system_flag = 0")
            for t in self.c.fetchall():
                tables.append(t[0].strip())
        except Exception as ex:
            log.error("Couldn't retrieve table list: %s." % ex)
        finally:
            self.c.close()
        return tables

    def getColumns(self, _table):
        columns = []
        self.c = self.conn.cursor()
        try:
            self.c.execute("select rdb$field_name from rdb$relation_fields where rdb$relation_name = '%s'" % (_table))
            for c in self.c.fetchall():
                columns.append(c[0].strip())
        except:
            log.error("Couldn't retrieve column info for table " + _table)
        finally:
            self.c.close()
        return columns

    def hasMetaInfo(self):
        return True
